<?php

/**
 * Abstract class heartbeataccess
 * This base class has final template methods which are
 * used by the derived concretes. The HeartbeatAccess is a
 * state object that is given to the HeartbeatMessageBuilder to
 * set the access to the current request.
 */
abstract class HeartbeatAccess {

  protected $_whoisuser_types = array(
    self::TYPE_ACTOR => 'Acting user',
    self::TYPE_USER_PROFILE => 'Requested user (from url)'
  );
  protected $_whoisuser_type = self::TYPE_ACTOR;
  protected $_uid = 0;
  protected $_heartbeatInfo = NULL;
  protected $_errors = array();
  protected $_has_errors = FALSE;
  protected $_offset_sql = 0;
  protected $_page = FALSE;

  const TYPE_ACTOR = 0;
  const TYPE_USER_PROFILE = 1;

  /**
   * Constructor
   * 
   * @param $type Array with the variable version of the requested object
   * @param $page Boolean to indicate if this is a page view
   */
  public function __construct($type, $page = FALSE) {
    global $user;

    $this->_page = $page;

    $this->_heartbeatInfo = new HeartbeatInfo();
    $this->_heartbeatInfo->name = $type['class'];
    $this->_heartbeatInfo->title = $type['name'];
    $this->_heartbeatInfo->module = $type['module'];
    $this->_heartbeatInfo->access = $type['access'];
    
    // Assign the who-is-user type
    $this->_whoisuser_type = variable_get('heartbeat_show_user_profile_messages_' . strtolower(get_class($this)), self::TYPE_ACTOR);

    if ($this->_whoisuser_type == self::TYPE_USER_PROFILE) {

      if(arg(0) == 'user' && is_numeric(arg(1)) && arg(1) > 0) {
        $this->_uid = arg(1);
      }
      else {
      	$this->_uid = $user->uid;
      }
    }
    else {
      $this->_uid = $user->uid;
    }
    // Populate the user object with a users relations
    $user->heartbeat_relations = heartbeat_get_related_uids($this->_uid);
    
    // Set info properties
    $this->_heartbeatInfo->uid = $this->_uid;
    $this->_heartbeatInfo->user_relations = $user->heartbeat_relations;
  }

  protected function setError($text) {
  	$this->_errors[] = $text;
  	$this->_has_errors = TRUE;
  }

  protected function hasAccess() {
    return TRUE;
  }

  public function hasErrors() {
  	return $this->_has_errors;
  }

  public function getErrors() {
    return $this->_errors;
  }

  public function getInfo() {
    return $this->_heartbeatInfo;
  }

  public function fetchMessages() {

    $heartbeat = $this->prepareHeartbeatInfo();

    // Call to a hook method to adapt sql and thus result
    $heartbeat = $this->dressUpMessages($heartbeat);

    return $this->hasAccess() ? $this->finishMessages($heartbeat) : array();
  }

  protected function resultSql($sql = '', $values = array()) {
    
  	$args = array();
    $args[] = $this->_heartbeatInfo->sql_start . $sql . $this->_heartbeatInfo->sql_end;
    $args[] = $this->_heartbeatInfo->language;
    $args[] = $this->_heartbeatInfo->offset_sql;
    // Limit the query from offset time to a month earlier
    $args[] = $this->_heartbeatInfo->offset_sql - 2678400; 
    $args = array_merge($args, $values);
    
    // The cron limits results to a maximum
    $result = call_user_func_array('db_query', $args);

  	$results = array();
    while($heartbeat_row = db_fetch_object($result) ) {
      $results[$heartbeat_row->uaid] = new HeartbeatActivity($heartbeat_row);
    }
    
    $stream = heartbeat_stream_load($this->getAccess());
    if (isset($stream['display_filters']) &&  $stream['display_filters'] == 1) {
    	$this->_heartbeatInfo->display_filters = TRUE;
    	$this->_heartbeatInfo->filters = $stream['filters'];
    }

    // Check if the number of results is really one more
    if (count($results) > $this->_heartbeatInfo->limit_sql) {
    	$this->_heartbeatInfo->pager = TRUE;
    }

    return $results;
  }

  protected function prepareHeartbeatInfo() {

    global $user, $language;

    $this->_heartbeatInfo->offset_sql = $this->_offset_sql;
   
    $this->_heartbeatInfo->limit_sql = variable_get($this->getAccess() . '_' . ($this->_page ? 'page' : 'block') . '_items_max', 250);
    $this->_heartbeatInfo->language = $language->language;

    // Sql parts
    $this->_heartbeatInfo->sql_start = "SELECT
      hm.message AS 'message_orig', hm.message_concat AS 'message_concat_orig',
      hm.attachments AS 'attachments', hm.concat_args, hm.perms, 
      ua.*, 
      1 AS 'count'
    FROM {heartbeat_activity} ua
    LEFT JOIN {heartbeat_messages} hm ON ua.message_id = hm.message_id
    WHERE 
      ua.language = '%s'
    AND 
      (ua.timestamp < %d AND ua.timestamp > %d )
    ";

    $this->_heartbeatInfo->sql_end = " ORDER BY timestamp DESC";

    $heartbeat = $this->createHeartbeatParser();

    return $heartbeat;
  }

  final protected function createHeartbeatParser() {
    $heartbeat = HeartbeatParser::instantiate($this->getAccess());
    return $heartbeat;
  }

  final public function setOffsetSql($offset = 0) {
  	if ($offset == 0) {
  	  $offset = $_SERVER['REQUEST_TIME'];
  	}
  	$this->_offset_sql = $offset;
  }

  protected function dressUpMessages(HeartbeatParser $heartbeat) { 
    $heartbeat->raw_messages = $this->resultSql();
  }

  protected function finishMessages(HeartbeatParser $heartbeat) {

    return $heartbeat;
  }

  public function addViewQuery(&$view) {
  	// Nothing for base class
  }

  final public function getAccess() {
  	return strtolower(get_class($this));
  }
}
